@@ -25,13 +25,16 @@ from typing import Any, Dict, List, Optional, Union, cast
 import boto3
 import botocore
 import pandas as pd
+import yaml
+from kubernetes import client
 from kubernetes import config as k8_config
+from kubernetes import dynamic
 from kubernetes import watch as k8_watch
 from kubernetes.client import *
-from slugify import slugify
 
 from aws_orbit_sdk.common import get_properties, get_stepfunctions_waiter_config
-from aws_orbit_sdk.common_pod_specification import TeamConstants
+
+# from aws_orbit_sdk.common_pod_specification import TeamConstants
 
 logging.basicConfig(
     format="%(asctime)s %(levelname)-8s %(message)s",
@@ -648,6 +651,18 @@ def list_running_cronjobs():
     return res["items"]
 
 
+def get_podsetting_spec(podsetting_name, team_name):
+    load_kube_config()
+    co = CustomObjectsApi()
+    crd = co.get_namespaced_custom_object("orbit.aws", "v1", team_name, "podsettings", podsetting_name)
+    return crd
+
+
+def get_priority(taskConfiguration: dict):
+    if "compute" in taskConfiguration and "priorityClassName" in taskConfiguration["compute"]:
+        return taskConfiguration["compute"]["priorityClassName"]
+
+
 def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str]) -> V1JobSpec:
     """
     Runs Task in Python in a notebook using lambda.
@@ -671,65 +686,46 @@ def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str]) -> V1J
         __CURRENT_ENV_MANIFEST__ = load_env_context_from_ssm(env_name)
 
     env = build_env(__CURRENT_ENV_MANIFEST__, env_name, taskConfiguration, team_name)
-    profile = resolve_profile(taskConfiguration)
-    image = resolve_image(__CURRENT_ENV_MANIFEST__, profile)
-    node_type = get_node_type(taskConfiguration)
+    env["AWS_ORBIT_ENV"] = props["AWS_ORBIT_ENV"]
+    env["AWS_ORBIT_TEAM_SPACE"] = props["AWS_ORBIT_TEAM_SPACE"]
+
+    priority_class_name = get_priority(taskConfiguration)
+    podsetting_name = resolve_podsetting_name(taskConfiguration)
+    podsetting_spec = get_podsetting_spec(podsetting_name, team_name)
+    image = resolve_image_from_podsetting(__CURRENT_ENV_MANIFEST__, podsetting_spec)
 
     job_name: str = f'run-{taskConfiguration["task_type"]}'
-    volumes: List[Dict[str, Any]] = list()
-    volume_mounts: List[Dict[str, Any]] = list()
     grant_sudo = False
-    if "kubespawner_override" in profile:
-        if "volumes" in profile["kubespawner_override"]:
-            volumes.extend(profile["kubespawner_override"]["volumes"])
-            _logger.info("profile override is attaching volumes: %s", volumes)
-        if "volume_mounts" in profile["kubespawner_override"]:
-            volume_mounts.extend(profile["kubespawner_override"]["volume_mounts"])
-            _logger.info("profile override is mounting volumes: %s", volume_mounts)
-    if "compute" in taskConfiguration:
-        if "grant_sudo" in taskConfiguration["compute"]:
-            if taskConfiguration["compute"]["grant_sudo"] or taskConfiguration["compute"]["grant_sudo"] == "True":
-                grant_sudo = True
-        if "volumes" in taskConfiguration["compute"]:
-            volumes.extend(taskConfiguration["compute"]["volumes"])
-            _logger.info("task override is attaching volumes: %s", volumes)
-        if "volume_mounts" in taskConfiguration["compute"]:
-            volume_mounts.extend(taskConfiguration["compute"]["volume_mounts"])
-            _logger.info("task override is mounting volumes: %s", volume_mounts)
-        if "labels" in taskConfiguration["compute"]:
-            labels = {**labels, **taskConfiguration["compute"]["labels"]}
-    node_selector: List[Dict[str, str]] = list()
-    _logger.info("volumes:%s", json.dumps(volumes))
-    _logger.info("volume_mounts:%s", json.dumps(volume_mounts))
+
+    labels = {**labels, **podsetting_spec["metadata"]["labels"]}
+    # MUST set this to the name of the podsetting in format 'orbit/{podsetting-name} so orbit-controller watcher will apply the settings
+    # MUST set 'app' to orbit-runner
+    label_indicator = "orbit/" + podsetting_spec["metadata"]["name"]
+    labels[label_indicator] = ""
+
+    labels["app"] = "orbit-runner"
 
     pod_properties: Dict[str, str] = dict(
         name=job_name,
-        image=image,
         cmd=["bash", "-c", "python /opt/python-utils/notebook_cli.py"],
         port=22,
+        image=image,
         service_account="default-editor",
-        node_selector=node_selector,
         run_privileged=False,
         allow_privilege_escalation=True,
         env=env,
-        volumes=volumes,
-        volume_mounts=volume_mounts,
+        priority_class_name=priority_class_name,
         labels=labels,
         logger=_logger,
         run_as_uid=1000,
         run_as_gid=100,
     )
 
-    if "kubespawner_override" in profile:
-        for k, v in profile["kubespawner_override"].items():
-            if k in ["image"]:
-                # special handling is already done for image
-                continue
-            if k in pod_properties:
-                raise RuntimeError("Override '%s' in profile is not allowed", k)
-            _logger.debug("profile overriding pod value %s=%s", k, v)
-            pod_properties[k] = v
-
+    ###
+    # 20210721 - podsettings are used, and the orbit-controller overrides the pod spec when created.
+    # The make_pod method has a lot of configurable items that are not used.  Orbit only needs the
+    # parameters to make a minimally viable pod spec.
+    ###
     pod: V1Pod = make_pod(**pod_properties)
     pod.spec.restart_policy = "Never"
     job_spec = V1JobSpec(
@@ -737,7 +733,6 @@ def _create_eks_job_spec(taskConfiguration: dict, labels: Dict[str, str]) -> V1J
         template=pod,
         ttl_seconds_after_finished=120,
     )
-
     return job_spec
 
 
@@ -961,6 +956,8 @@ def make_pod(
             for secret_ref in image_pull_secrets
         ]
 
+    if priority_class_name:
+        pod.spec.priority_class_name = priority_class_name
     if node_selector:
         pod.spec.node_selector = node_selector
 
@@ -1155,7 +1152,27 @@ def make_pod(
     return pod
 
 
+def get_podsetting_spec(podsetting_name, team_name):
+    # /apis/orbit.aws/v1/namespaces/lake-user/podsettings/orbit-custom-image-with-apps-lake-user
+    load_kube_config()
+    co = CustomObjectsApi()
+    return co.get_namespaced_custom_object("orbit.aws", "v1", team_name, "podsettings", podsetting_name)
+
+
+def resolve_image_from_podsetting(__CURRENT_ENV_MANIFEST__, podsetting_spec):
+    if podsetting_spec and podsetting_spec["spec"] and podsetting_spec["spec"]["image"]:
+        image = podsetting_spec["spec"]["image"]
+    else:
+        repository = (
+            f'{__CURRENT_ENV_MANIFEST__["Images"]["JupyterUser"]["Repository"]}:'
+            f'{__CURRENT_ENV_MANIFEST__["Images"]["JupyterUser"]["Version"]}'
+        )
+        image = f"{repository}"
+    return image
+
+
 def resolve_image(__CURRENT_ENV_MANIFEST__, profile):
+
     if not profile or "kubespawner_override" not in profile or "image" not in profile["kubespawner_override"]:
         repository = (
             f'{__CURRENT_ENV_MANIFEST__["Images"]["JupyterUser"]["Repository"]}:'
@@ -1178,19 +1195,9 @@ def build_env(__CURRENT_ENV_MANIFEST__, env_name, taskConfiguration, team_name):
     return env
 
 
-def resolve_profile(taskConfiguration):
-    team_constants = TeamConstants()
-    if "compute" in taskConfiguration and "profile" in taskConfiguration["compute"]:
-        profile_name = taskConfiguration["compute"]["profile"]
-        profile_name = slugify(profile_name)
-        _logger.info(f"using profile %s", profile_name)
-        profile = team_constants.profile(profile_name)
-        if not profile:
-            raise Exception("Profile '%s' not found", profile)
-    else:
-        profile = team_constants.default_profile()
-        _logger.info(f"using default profile %s", profile)
-    return profile
+def resolve_podsetting_name(taskConfiguration):
+    if "compute" in taskConfiguration and "podsetting" in taskConfiguration["compute"]:
+        return taskConfiguration["compute"]["podsetting"]
 
 
 def _run_task_eks(taskConfiguration: dict) -> Any:
@@ -1639,3 +1646,102 @@ def all_tasks_stopped(tasks_state: Any) -> bool:
         if t["lastStatus"] in ("PENDING", "RUNNING"):
             return False
     return True
+
+
+def build_podsetting(env_name: str, team_name: str, podsetting: str, debug: bool) -> None:
+    ps = json.loads(podsetting)
+    if not ps["description"] or not ps["name"]:
+        raise Exception(f"Podsetting name and description not present")
+    podsetting_name = ps["name"]
+    spec_template = _generate_podsetting_spec_base(
+        podsetting_name=podsetting_name, description=ps["description"], env_name=env_name, team_name=team_name
+    )
+    spec = yaml.load(spec_template, Loader=yaml.SafeLoader)
+    env_params = ps.get("env", None)
+    if env_params:
+        spec["spec"]["env"] = env_params
+
+    image = ps.get("image", None)
+    if image:
+        spec["spec"]["image"] = image
+
+    resources = ps.get("resources", None)
+    if resources is not None:
+        spec["spec"]["resources"] = resources
+
+    nodegroup = ps.get("node-group", None)
+    instancetype = ps.get("instance-type", None)
+    if nodegroup or instancetype:
+        spec["spec"]["nodeSelector"] = {}
+        if nodegroup:
+            spec["spec"]["nodeSelector"]["orbit/node-group"] = nodegroup
+        if instancetype:
+            spec["spec"]["nodeSelector"]["node.kubernetes.io/instance-type"] = instancetype
+
+    _logger.debug(yaml.dump(spec))
+
+    dynamic_client = _dynamic_client()
+    # This is a hack....we cannot update/patch so delete and create (for now)
+    try:
+        _destroy_podsetting(namespace=team_name, podsetting_name=podsetting_name, client=dynamic_client)
+        time.sleep(3)
+    except ApiException as e:
+        _logger.info(f"Tried to delete {podsetting_name} but that podsetting was not found...moving on")
+
+    _deploy_podsetting(namespace=team_name, name=podsetting_name, client=dynamic_client, podsetting_spec=spec)
+    _logger.info(f"PodSetting {podsetting_name} deployed to {team_name}")
+
+
+def delete_podsetting(namespace: str, podsetting_name: str) -> None:
+    try:
+        _destroy_podsetting(namespace, podsetting_name, client=_dynamic_client())
+    except ApiException as e:
+        _logger.info(f"PodSetting {podsetting_name} failed to delete - : {e}")
+
+
+def _deploy_podsetting(
+    namespace: str, name: str, client: dynamic.DynamicClient, podsetting_spec: Dict[str, Any]
+) -> None:
+    ORBIT_API_VERSION = os.environ.get("ORBIT_API_VERSION", "v1")
+    ORBIT_API_GROUP = os.environ.get("ORBIT_API_GROUP", "orbit.aws")
+    api = client.resources.get(api_version=ORBIT_API_VERSION, group=ORBIT_API_GROUP, kind="PodSetting")
+    api.create(namespace=namespace, body=podsetting_spec)
+
+
+def _destroy_podsetting(namespace: str, podsetting_name: str, client: dynamic.DynamicClient) -> None:
+    ORBIT_API_VERSION = os.environ.get("ORBIT_API_VERSION", "v1")
+    ORBIT_API_GROUP = os.environ.get("ORBIT_API_GROUP", "orbit.aws")
+    api = client.resources.get(api_version=ORBIT_API_VERSION, group=ORBIT_API_GROUP, kind="PodSetting")
+    api.delete(namespace=namespace, name=podsetting_name, body={})
+
+
+def _dynamic_client() -> dynamic.DynamicClient:
+    load_kube_config()
+    return dynamic.DynamicClient(client=api_client.ApiClient())
+
+
+def _generate_podsetting_spec_base(podsetting_name, description, env_name, team_name):
+    spec_template = """
+        kind: PodSetting
+        apiVersion: orbit.aws/v1
+        metadata:
+            labels:
+                orbit/env: {env_name}
+                orbit/space: team
+                orbit/team: {team_name}
+            name: {podsetting_name}
+            namespace: {team_name}
+        spec:
+            containerSelector:
+                jsonpath: metadata.labels.app
+            desc: {description}
+            podSelector:
+                matchExpressions:
+                - key: orbit/{podsetting_name}
+                  operator: Exists
+            securityContext:
+                runAsUser: 1000
+    """.format(
+        team_name=team_name, description=description, env_name=env_name, podsetting_name=podsetting_name
+    )
+    return spec_template
